// IDAutomationPPC.Drawing.GdiGraphics.cs - Supports native drawing
// with GDI drawing functions.
//
// Code from _Programming the .NET Compact Framework with C#_
// and _Programming the .NET Compact Framework with VB_
// (c) Copyright 2002-2004 Paul Yao and David Durant. 
// All rights reserved.
using System;
using System.Runtime.InteropServices;

namespace IDAutomationPPC.Drawing
{
   /// <summary>
   /// Summary description for IDAutomationPPC.
   /// </summary>
   public class GdiGraphics
   {
	   const uint  LMEM_FIXED  = 0x0000;
	   const uint LMEM_ZEROINIT = 0x0040;
	   public const uint LPTR = (LMEM_FIXED | LMEM_ZEROINIT);


	   [DllImport("coredll.dll")]
	   internal static extern IntPtr LocalAlloc(
		   uint uFlags,
		   uint uBytes );

	   [DllImport("coredll.dll")]
	   internal static extern IntPtr LocalFree(IntPtr hMem); 

      [DllImport("coredll.dll")]
      public static extern IntPtr GetDC (IntPtr hWnd);

      [DllImport("coredll.dll")]
      public static extern int ReleaseDC (IntPtr hWnd, IntPtr hDC);

      [DllImport("coredll.dll")]
      public static extern int GetDeviceCaps (IntPtr hdc, CAPS iIndex);

      [DllImport("coredll.dll", EntryPoint="ExtTextOutW")]
      public static extern int 
         Real_ExtTextOut (IntPtr hdc, int X, int Y, int fuOptions, 
         IntPtr lprc, string lpString, int cbCount, IntPtr lpDx);

	   [DllImport("Gdi32.dll", SetLastError = true)]
	   private static extern int SetStretchBltMode(IntPtr hdc, int iStretchMode);

	   /// <summary>
	   /// Draws a bitmap on the device context
	   /// </summary>
	   /// <param name="hdc">handle to device context we are writing to</param>
	   /// <param name="XDest">xcoordinate to begin drawing on DC</param>
	   /// <param name="YDest">ycoordinate to begin drawing on DC</param>
	   /// <param name="nDestWidth">The width of the image on DC</param>
	   /// <param name="nDestHeight">The height of the image on DC</param>
	   /// <param name="XSrc">xcoordinate in bitmap to begin pulling bits from</param>
	   /// <param name="YSrc">ycoordinate in bitmap to begin pulling bits from</param>
	   /// <param name="nSrcWidth">The width of the source bitmap</param>
	   /// <param name="nSrcHeight">The width of the source bitmap</param>
	   /// <param name="lpBits">array of bits representing the bitmap</param>
	   /// <param name="lpBitsInfo"></param>
	   /// <param name="iUsage">Specifies whether the bmiColors member of the BITMAPINFO structure was provided and,
	   ///  if so, whether bmiColors contains explicit red, green, blue 
	   ///  (RGB) values or indexes. The iUsage parameter must be one of the following values.
	   ///  DIB_PAL_COLORS 	The array contains 16-bit indexes into the logical palette of the source device context.
	   ///  DIB_RGB_COLORS 	The color table contains literal RGB values.</param>
	   /// <param name="dwRop">Specifies how the source pixels, the destination device context's current brush, and the destination pixels are to be combined to form the new image.</param>
	   /// <returns></returns>
	   [DllImport("coredll.dll")]
	   public static extern int StretchDIBits(IntPtr hdc, int XDest, int YDest,
		   int nDestWidth, int nDestHeight, int XSrc, int YSrc, int nSrcWidth,
		   int nSrcHeight, byte [] lpBits, IntPtr lpBitsInfo, uint iUsage,
		   uint dwRop);
	   
      [DllImport("coredll.dll")]
      public static extern int DeleteObject (IntPtr hObject);

      [DllImport("coredll.dll")]
      public static extern 
         IntPtr SelectObject (IntPtr hdc, IntPtr hgdiobj);

      [DllImport("coredll.dll")]
      public static extern 
         int SetBkMode (IntPtr hdc, BKMODE bkmode);

      public static 
         int ExtTextOut(IntPtr hdc, int X, int Y, int fuOptions,
         IntPtr lprc, string lpString, int cbCount, IntPtr lpDx)
      {
         // "Transparent" only touches foreground pixels.
         SetBkMode(hdc, BKMODE.TRANSPARENT);
         
         return Real_ExtTextOut(hdc, X, Y, fuOptions, lprc, lpString, cbCount, lpDx);
      }
	   public const uint DIB_RGB_COLORS = 0;
	   public const uint SRCCOPY = 0xcc0020;
	   public const uint SRCAND = 0x008800C6;
	   public const int BI_RGB = 0;
	   public const int COLORONCOLOR = 3;
	   public const int HALFTONE = 4;
	   
   }  // class
	
	// BITMAPINFO with embedded BITMAPINFOHEADER to simplify the code
	public struct BITMAPINFO
	{
		public int biSize;
		public int biWidth;
		public int biHeight;
		public short biPlanes;
		public short biBitCount;
		public int biCompression;
		public int biSizeImage;
		public int biXPelsPerMeter;
		public int biYPelsPerMeter;
		public int biClrUsed;
		public int biClrImportant;

		public int bmiColors;
	};

	struct RGBQUAD
	{
		public byte rgbBlue;
		public byte rgbGreen;
		public byte rgbRed;
		public byte rgbReserved;
	};	

   public enum BKMODE
   {
      TRANSPARENT = 1,
      OPAQUE = 2
   }

   public enum CAPS
   {
      HORZRES =  8,          // Horizontal width in pixels
      VERTRES = 10,          // Vertical height in pixels
      LOGPIXELSY = 90,
      PHYSICALWIDTH   = 110, // Physical Width in device units   
      PHYSICALHEIGHT  = 111, // Physical Height in device units  
      PHYSICALOFFSETX = 112, // Physical Printable Area x margin 
      PHYSICALOFFSETY = 113, // Physical Printable Area y margin 
   }

}  // namespace
